<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\User;
use Illuminate\Http\Request;

class AdminsController extends Controller
{


    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $rows = User::whereType('admin')->orderByDesc('id')->get();
        return view('admin.admin.index', compact('rows'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.admin.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'email' => 'required|email|unique:users',
            'mobile' => 'required|unique:users,mobile',
            'password' => 'required|min:6',
            'status' => 'required|in:active,not_active'
        ]);
        $admin = new User();
        $admin['name'] = $request['name'];
        $admin['email'] = $request['email'];
        $admin['mobile'] = $request['mobile'];
        $admin['status'] = $request['status'];
        $admin['password'] = bcrypt($request['password']);
        $admin['type'] = 'admin';
        $admin->save();
        toast('تمت العملية بنجاح', 'success');
        return redirect()->route('admin.index');


    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $row = User::find($id);
        return view('admin.admin.edit', compact('row'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
         $this->validate($request, [
            'name' => 'required',
            'email' => 'required|email|unique:users,email,' . $id,
            'mobile' => 'required|unique:users,mobile,' . $id,
             'status' => 'required|in:active,not_active'
        ]);
         $admin = User::find($id);
        $admin['name'] = $request['name'];
        $admin['email'] = $request['email'];
        $admin['status'] = $request['status'];
        $admin['mobile'] = $request['mobile'];

        if ($request['password']) $admin['password'] = bcrypt($request['password']);

        $admin['type'] = 'admin';
        $admin->update();
        toast('تمت العملية بنجاح','success');
        return redirect()->route('admin.index');

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        User::destroy($id);
        return redirect()->back()->with('message', 'تم الحذف بنجاح');
    }
}
