<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Order;
use App\Models\Service;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;

class BarbersController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $rows = User::whereType('barber')->where('admin_accept', 'true')->orderByDesc('id')->get();
        return view('admin.barber.index', compact('rows'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.barber.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
//        $this->validate($request,
//        [
//            'name'=>'required',
//            'last_name'=>'required',
//            'email'=>'required|email|unique:users,email',
//            'mobile'=>'required|unique:users,mobile',
//            'password'=>'required',
//
//        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $barber = User::find($id);
        $rows = Order::where('status', '!=', 'in_completed')->where('barber_id', $id)->whereDate('created_at', '=', Carbon::today()->toDateString())->orderByDesc('id')->get();
        $services = Service::all();
        return view('admin.barber.show', compact('barber', 'rows', 'services'));
    }

    public function barber_order_search(Request $request)
    {
        $this->validate($request, [
            'barber_id' => 'required'
        ]);
        $rows = Order::where('status', '!=', 'in_completed')->where('barber_id',$request['barber_id'])->when(($request['service_id'] != "0"), function ($order) use ($request) {
            $order->whereHas('services', function ($q) use ($request) {
                $q->where('service_id', $request['service_id']);
            });
        });

        if ($request['start'] && $request['end']) {

            if ($request['start'] == $request['end']) {
                $rows = $rows->whereDate('created_at', '>=', Carbon::parse($request['start'])->toDateString())->where('barber_id',$request['barber_id']);

            } else {
                $rows = $rows->whereBetween('created_at', [Carbon::parse($request['start']), Carbon::parse($request['end'])])->where('barber_id',$request['barber_id']);

            }
        } elseif ($request['start'] && $request['end'] == null) {

            $rows = $rows->where('created_at', '>=', Carbon::parse($request['start']))->where('barber_id',$request['barber_id']);
        } elseif ($request['end'] && $request['start'] == null) {
            $rows = $rows->where('created_at', '<=', Carbon::parse($request['end']))->where('barber_id',$request['barber_id']);
        }


        $rows = $rows->get();
        $barber = User::find($request['barber_id']);


        $services = Service::all();
        return view('admin.barber.show', compact('barber', 'rows', 'services'));


    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'status' => 'in:active,not_active'
        ]);
        User::where('id', $id)->update([
            'status' => $request['status']
        ]);
        toast('تم تغيير الحالة بنجاح', 'success');
        return redirect()->back();
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        User::destroy($id);
        toast('تمت العملية بنجاح ', 'success', 'top-left');
        return back();
    }
}
