<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Offer;
use App\User;
use Illuminate\Http\Request;

class OffersController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $rows = Offer::orderByDesc('id')->get();
        return view('admin.offers.index', compact('rows'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $users = User::whereType('client')->get();
        return view('admin.offers.create', compact('users'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'body_ar' => 'required',
            'body_en' => 'required',
            'discount' => 'required',
            'code' => 'required',
            'to' => 'required|in:all,users'
        ]);

        if ($request['to'] == 'all') {
            $offer = new Offer();
            $offer['body->ar'] = $request['body_ar'];
            $offer['body->en'] = $request['body_en'];
            $offer['discount'] = $request['discount'];
            $offer['code'] = $request['code'];
            $offer->save();
        } else {
            foreach ($request['users'] as $user) {
                $offer = new Offer();
                $offer['body->ar'] = $request['body_ar'];
                $offer['body->en'] = $request['body_en'];
                $offer['discount'] = $request['discount'];
                $offer['code'] = $request['code'];
                $offer['user_id'] = $user;
                $offer->save();
            }
        }

        toast('تمت العمليه بنجاح', 'success', 'top-left');
        return redirect()->route('offers.index');


    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request,[
            'code'=>'required',
            'discount'=>'required',
            'body_ar'=>'required',
            'body_en'=>'required'
        ]);
        $offer = Offer::find($id);
        $offer['body->ar'] = $request['body_ar'];
        $offer['body->en'] = $request['body_en'];
        $offer['code'] = $request['code'];
        $offer['discount'] = $request['discount'];
        $offer->update();
        toast('تمت العمليه بنجاح','success');
        return  back();
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        Offer::destroy($id);
        toast('تمت العمليه بنجاح','success');
        return  back();
    }
}
