<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Order;
use App\Models\OrderService;
use App\Models\Service;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;

class OrdersController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $rows = Order::where('status','!=','in_completed')->whereDate('created_at', '=', Carbon::today()->toDateString())->orderByDesc('id')->get();
        $barbers = User::whereType('barber')->get();
        $users = User::whereType('client')->get();
        $services = Service::all();
        return view('admin.orders.index', compact('rows', 'users', 'barbers', 'services'));
    }

    public function order_search(Request $request)
    {

         $rows = Order::where('status','!=','in_completed')->when(($request['service_id'] != "0"), function ($order) use ($request) {
             $order->whereHas('services', function ($q) use ($request) {
                $q->where('service_id', $request['service_id']);
            });
        });

        if ($request['start'] && $request['end']) {

            if ($request['start'] == $request['end']){
                $rows = $rows->whereDate('created_at', '>=', Carbon::parse($request['start'])->toDateString());

            }else{
                $rows = $rows->whereBetween('created_at', [Carbon::parse($request['start']), Carbon::parse($request['end'])]);

            }
        } elseif ($request['start'] && $request['end'] == null) {

            $rows = $rows->where('created_at', '>=', Carbon::parse($request['start']));
        } elseif ($request['end'] && $request['start'] == null) {
            $rows = $rows->where('created_at', '<=', Carbon::parse($request['end']));
        }
        if ($request['user_id'] != '0') {
            $rows = $rows->where('user_id', '==', $request['user_id']);

        }
        if ($request['barber_id'] != '0') {
            $rows = $rows->where('barber_id', '==', $request['barber_id']);
        }

        $rows = $rows->get();

        $barbers = User::whereType('barber')->get();
        $users = User::whereType('client')->get();
        $services = Service::all();
        return view('admin.orders.index', compact('rows', 'barbers', 'users', 'services'));


    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $row = Order::find($id);
        $order_services = OrderService::where('order_id', $row->id)->get();

        return view('admin.orders.show', compact('row', 'order_services'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        Order::destroy($id);
        toast('تمت العملية بنجاح', 'success');
        return back();
    }
}
