<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\BarberOrderResource;
use App\Http\Resources\BarberServices;
use App\Http\Resources\OrderResource;
use App\Http\Resources\ServicesResource;
use App\Models\Address;
use App\Models\Coupon;
use App\Models\Notification;
use App\Models\Offer;
use App\Models\Order;
use App\Models\OrderService;
use App\Models\Service;
use App\Models\Setting;
use App\Models\UserServices;
use App\Models\UserToken;
use App\User;
use Carbon\Carbon;
use Edujugon\PushNotification\PushNotification;
use Illuminate\Http\Request;
use JWTAuth;
use Lcobucci\JWT\Token;

class ServicesController extends Controller
{
    public function index(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $rows = Service::paginate(20);
        return ServicesResource::collection($rows)->additional(['value' => true, 'code' => 200]);
    }

    public function orders($flag)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        if ($flag == 'current') {
            $rows = Order::where('user_id', $user->id)->where('status', '!=', 'finish')->where('status', '!=', 'in_completed')->where('status', '!=', 'canceled')->orderByDesc('id')->where('user_delete', 'false')->paginate(10);
        } else {
            $rows = Order::where('user_id', $user->id)->where('user_delete', 'false')->where('status', '!=', 'in_completed')->whereIn('status', ['finish', 'canceled'])->orderByDesc('id')->paginate(10);
        }
        return OrderResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function delete_order($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }


        if ($user->type == 'client') {
            Order::where('id', $id)->update([
                'user_delete' => 'true'
            ]);
            $rows = Order::where('user_id', $user->id)->where('user_delete', 'false')->where('status', '!=', 'in_completed')->whereIn('status', ['finish', 'canceled'])->orderByDesc('id')->paginate(10);
            return OrderResource::collection($rows)->additional(['value' => true, 'code' => 200]);

        } else {
            Order::where('id', $id)->update([
                'barber_delete' => 'true'
            ]);
            $rows = Order::where('barber_id', $user->id)->where('barber_delete', 'false')->where('status', '!=', 'in_completed')->orderByDesc('id')->paginate(10);

            return OrderResource::collection($rows)->additional(['value' => true, 'code' => 200]);


        }


    }

    public function order_services(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'address_id' => 'required_without:address|exists:addresses,id',
            'services_id' => 'required',
            'services_counts' => 'required',
            'lat' => 'required_without:address_id',
            'lng' => 'required_without:address_id',
            'address' => 'required_without:address_id',
        ]);


        if ($validator->passes()) {

            $user_address = $request['address_id'];
            if ($request['lat'] || $request['lng'] || $request['address']) {

                $check = Address::where(['lat' => $request['lat'], 'lng' => $request['lng']])->first();
                if ($check) {
                    $user_address = $check->id;
                } else {
                    $new_address = Address::create([
                        'lat' => $request['lat'],
                        'lng' => $request['lng'],
                        'address' => $request['address'],
                        'user_id' => $user->id
                    ]);
                    $user_address = $new_address->id;
                }

            }


            $lat = $request['lat'];
            $lng = $request['lng'];


            $all_barbers = User::select(\DB::raw('*, ( 6367 * acos( cos( radians(' . $lat . ') ) * cos( radians( lat ) ) * cos( radians( lng ) - radians(' . $lng . ') ) + sin( radians(' . $lat . ') ) * sin( radians( lat ) ) ) ) AS distance'))
                ->having('distance', '<', 5)
                ->orderBy('distance')->where('type', 'barber')->where('notifications', 'true')
                ->where('admin_accept', 'true')->where('accept_orders', 'true')->where('status', 'active')
                ->get();


            $barbers = [];
            foreach ($all_barbers as $single_barber) {
                $barber_orders = Order::where('barber_id', $single_barber->id)->where('status', '!=', 'finish')->where('status', '!=', 'canceled')->count();
                if ($barber_orders > 0) {
                    //
                } else {
                    $barbers[] = $single_barber->id;
                }
            }
//            if (count($barbers) < 1) return response()->json(['value' => false, 'msg' => \Lang::get('data.no_barbers'), 'code' => 422]);


            $user_orders = Order::where('user_id', $user->id)->where('status', '!=', 'finish')->where('status', '!=', 'canceled')->count();
            if ($user_orders > 0) return response()->json(['value' => false, 'msg' => \Lang::get('data.orders_exists'), 'code' => 200]);


            $setting = Setting::first();
            $order = new Order();
            $order['user_id'] = $user->id;
            $order['address_id'] = $user_address;
//            $order['fees'] = $setting ? $setting->fee : 0;
            $order['delivery_fee'] = $setting ? $setting->delivery_fee : 0;
            $order['order_code'] = uniqid();
            $order['status'] = "in_completed";
            $order->save();
            $service_ids = explode(',', $request['services_id']);
            $service_counts = explode(',', $request['services_counts']);


            $services_price = 0;

            $counter = 0;


            foreach ($service_ids as $item) {

                OrderService::create([
                    'order_id' => $order->id,
                    'service_id' => $service_ids[$counter],
                    'count' => $service_counts[$counter]
                ]);
                $single_service = Service::find($service_ids[$counter]);
                $services_price += $single_service->price * $service_counts[$counter];
                $counter++;
            }


            $order->services_price = $services_price;
            if ($services_price >= 150){

                $services_price = $services_price - ( $services_price * $setting->discount_fee / 100);

            }
            $order['fees'] = $services_price * $setting->fee / 100 ; // 3

            $order->total_price = $services_price + $order->fees + $order->delivery_fee;
            $order->service_price_fee = $services_price + $order->fees + $order->delivery_fee;



            //dd($services_price , $order->total_price , $order->fees , $order->service_price_fee ,$order->service_price_fee - $order->total_price);
            $order->update();

//            $group_discount = $setting ? $setting->delivery_fee : 100;


//            if ($order->total_price >= 150) {
//                $order->total_price = $order->total_price - ($order->total_price * $setting->discount_fee / 100);
//                $order->update();
//            }


            return response()->json(['value' => true, 'data' => (string)$order->id, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function complete_order(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'order_id' => 'required|exists:orders,id',
            'payment_type' => 'required|in:visa,cash'
        ]);
        if ($validator->passes()) {
            $order = Order::find($request['order_id']);
            $order['notes'] = $request['notes'];
            $order['status'] = 'search';
            $order['payment_type'] = $request['payment_type'];
            if ($request['discount_code']) {
                $check = Offer::where('code', $request['discount_code'])->where('available','true')->first();
                if ($check) {
                    $order['discount_code'] = $request['discount_code'];
                    $order['total_price'] = $order->total_price - ($order->total_price * $check->discount / 100);
                    $order->update();
                    if ($check->user_id == $user->id){
                        $check->available = 'false';
                        $check->update();
                    }


                } else {
                    return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_discount_code'), 'code' => 422]);
                }
            }
            $order->update();


            $address = Address::find($order->address_id);
            $lat = $address->lat;
            $lng = $address->lng;


            $all_barbers = User::select(\DB::raw('*, ( 6367 * acos( cos( radians(' . $lat . ') ) * cos( radians( lat ) ) * cos( radians( lng ) - radians(' . $lng . ') ) + sin( radians(' . $lat . ') ) * sin( radians( lat ) ) ) ) AS distance'))
                ->having('distance', '<', 5)
                ->orderBy('distance')->where('type', 'barber')->where('notifications', 'true')
                ->where('admin_accept', 'true')->where('accept_orders', 'true')->where('status', 'active')
                ->get();


            $barbers = [];
            foreach ($all_barbers as $single_barber) {
                $barber_orders = Order::where('barber_id', $single_barber->id)->where('status', '!=', 'finish')->where('status', '!=', 'canceled')->count();
                if ($barber_orders > 0) {
                    //
                } else {
                    $barbers[] = $single_barber->id;
                }
            }

            foreach ($barbers as $barber) {

                $notification = new Notification();
                $notification['user_id'] = $barber;
                $notification['body->ar'] = 'لديك طلب جديد';
                $notification['body->en'] = 'You have a new order';
                $notification['order_id'] = $order->id;
                $notification['type'] = 'search';
                $notification['seen'] = 'false';
                $notification->save();
            }


            $tokens = UserToken::whereIn('user_id', $barbers)->get();
            foreach ($tokens as $token) {


                if ($token->lang == 'ar') {
                    $title = 'لديك طلب جديد';
                    $body = "لديك طلب جديد";
                } else {

                    $title = 'You Have a new order';
                    $body = "You Have a new order";

                }

                $push = new PushNotification('fcm');


                if ($token->device_type == 'ios') {
                    $msg = [
                        'data' => [
                            'title_ar' => "طلب جديد",
                            'title_en' => "New Order",
                            'body_ar' => "لديك طلب جديد",
                            'body_en' => "You have a new order",
                            'type' => 'search',
                            'sound' => 'default',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'notification' => [
                            'title_ar' => "طلب جديد",
                            'title_en' => "New Order",
                            'body_ar' => "لديك طلب جديد",
                            'body_en' => "You have a new order",
                            'type' => 'search',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,

                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                } else {
                    $msg = [
                        'data' => [
                            'title_ar' => "طلب جديد",
                            'title_en' => "New Order",
                            'body_ar' => "لديك طلب جديد",
                            'body_en' => "You have a new order",
                            'type' => 'search',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,

                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                }


            }


            $data = new OrderResource($order);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200,]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function single_order($id)
    {
        $orders = Order::paginate(20);
        return OrderResource::collection($orders)->additional(['value' => true, 'code' => 200]);

    }

    public function barber_orders()
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $rows = Order::where('barber_id', $user->id)->where('status', '!=', 'in_completed')->where('barber_delete', 'false')->orderByDesc('id')->paginate(10);

        return OrderResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function service(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [
            'service_id' => 'required|exists:services,id'
        ]);
        if ($validator->passes()) {

            $check = UserServices::where(['user_id' => $user->id, 'service_id' => $request['service_id']])->first();
            if ($check) {
                $check->delete();
                return response()->json(['value' => true, 'data' => \Lang::get('data.remove_service'), 'code' => 200]);
            } else {
                UserServices::create(['user_id' => $user->id, 'service_id' => $request['service_id']]);
                return response()->json(['value' => true, 'data' => \Lang::get('data.add_service'), 'code' => 200]);

            }

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }

    }

    public function services()
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }

        $services = Service::paginate(10);

        return BarberServices::collection($services)->additional(['value' => true, 'code' => 200]);


    }

    public function resend_order_notification($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }

        $order = Order::find($id);
        if (!$order) return response()->json(['value' => false, 'msg' => 'wrong id for order', 'code' => 422]);
        $order->created_at = Carbon::now();
        $order->update();

        $address = Address::find($order->address_id);
        $lat = $address->lat;
        $lng = $address->lng;
        $max_distance = 5;
        $lat_small = $lat - ($max_distance * 0.018);
        $lat_large = $lat + ($max_distance * 0.018);
        $lng_small = $lng - ($max_distance * 0.018);
        $lng_large = $lng + ($max_distance * 0.018);
        $barbers = User::whereType('barber')->where('notifications', 'true')
            ->where('admin_accept', 'true')->where('accept_orders', 'true')->where('status', 'active')
            ->whereBetween('lat', [$lat_small, $lat_large])->whereBetween('lng', [$lng_small, $lng_large])
            ->pluck('id');


        foreach ($barbers as $barber) {

            $notification = new Notification();
            $notification['user_id'] = $barber;
            $notification['body->ar'] = 'لديك طلب جديد';
            $notification['body->en'] = 'You have a new order';
            $notification['order_id'] = $order->id;
            $notification['type'] = 'search';
            $notification['seen'] = 'false';
            $notification->save();
        }


        $tokens = UserToken::whereIn('user_id', $barbers)->get();
        foreach ($tokens as $token) {
            if ($token->lang == 'ar') {
                $title = 'لديك طلب جديد';
                $body = "لديك طلب جديد";
            } else {

                $title = 'You Have a new order';
                $body = "You Have a new order";

            }
            $push = new PushNotification('fcm');


            if ($token->device_type == 'ios') {
                $msg = [
                    'data' => [
                        'title_ar' => "طلب جديد",
                        'title_en' => "New Order",
                        'body_ar' => "لديك طلب جديد",
                        'body_en' => "You have a new order",
                        'type' => 'search',
                        'sound' => 'default',
                        'title' => $title,
                        'body' => $body,
                        'order_id' => $order->id,

                        'click_action' => 'FCM_PLUGIN_ACTIVITY',
                    ],
                    'notification' => [
                        'title_ar' => "طلب جديد",
                        'title_en' => "New Order",
                        'body_ar' => "لديك طلب جديد",
                        'body_en' => "You have a new order",
                        'type' => 'search',
                        'title' => $title,
                        'body' => $body,
                        'order_id' => $order->id,

                        'sound' => 'default',
                        'click_action' => 'FCM_PLUGIN_ACTIVITY',
                    ],
                    'priority' => 'high',
                ];
                $push->setMessage($msg)
                    ->setDevicesToken($token->token)
                    ->send();

            } else {
                $msg = [
                    'data' => [
                        'title_ar' => "طلب جديد",
                        'title_en' => "New Order",
                        'body_ar' => "لديك طلب جديد",
                        'body_en' => "You have a new order",
                        'type' => 'search',
                        'title' => $title,
                        'body' => $body,
                        'order_id' => $order->id,
                        'sound' => 'default',
                        'click_action' => 'FCM_PLUGIN_ACTIVITY',
                    ],
                    'priority' => 'high',
                ];
                $push->setMessage($msg)
                    ->setDevicesToken($token->token)
                    ->send();

            }


        }


        $data = new OrderResource($order);
        return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
    }

    public function order_details($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $order = Order::find($id);
        if (!$order) return response()->json(['value' => false, 'msg' => 'wrong id', 'code' => 422]);

        $data = new OrderResource($order);
        return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
    }

    public function single_order_details($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $order = Order::find($id);
        if (!$order) return response()->json(['value' => false, 'msg' => 'wrong id', 'code' => 422]);

        $data = new BarberOrderResource($order);
        return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
    }

    public function cancel_order($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $order = Order::find($id);
        if ($order) {
            $order->status = 'canceled';
            $order->update();
            $users_id = [];
            $users_id[] = $order->user_id;
            if ($order->barber_id) $users_id[] = $order->barber_id;
            $tokens = UserToken::whereIn('user_id', $users_id)->whereStatus('connected')->get();
            foreach ($tokens as $token) {
                if ($token->lang == 'ar') {
                    $title = 'تم إلغاء الطلب';
                    $body = "تم إلغاء الطلب";
                } else {

                    $title = 'The order canceled';
                    $body = "The order canceled";

                }
                $push = new PushNotification('fcm');
                $notification = new Notification();
                $notification['user_id'] = $token->user_id;
                $notification['body->ar'] = 'تم إلغاء الطلب';
                $notification['body->en'] = 'The order canceled';
                $notification['order_id'] = $order->id;
                $notification['type'] = 'canceled';
                $notification['seen'] = 'false';
                $notification->save();

                if ($token->device_type == 'ios') {
                    $msg = [
                        'data' => [
                            'title_ar' => "تم إلغاء الطلب",
                            'title_en' => "The order canceled",
                            'body_ar' => "تم إلغاء الطلب",
                            'body_en' => "The order canceled",
                            'type' => 'canceled',
                            'sound' => 'default',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,

                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'notification' => [
                            'title_ar' => "تم إلغاء الطلب",
                            'title_en' => "The order canceled",
                            'body_ar' => "تم إلغاء الطلب",
                            'body_en' => "The order canceled",
                            'type' => 'canceled',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,

                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                } else {
                    $msg = [
                        'data' => [
                            'title_ar' => "تم إلغاء الطلب",
                            'title_en' => "The order canceled",
                            'body_ar' => "تم إلغاء الطلب",
                            'body_en' => "The order canceled",
                            'type' => 'canceled',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,

                            'sound' => 'default',
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                }


            }


            $rows = Order::where('user_id', $user->id)->orderByDesc('id')->paginate(10);


//            return OrderResource::collection($rows)->additional(['value' => true, 'code' => 200]);
            return response()->json(['value' => true, 'data' => \Lang::get('data.canceled'), 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => 'wrong id', 'code' => 422]);
        }

    }
}
