<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;

use App\Http\Resources\AddressesResource;
use App\Http\Resources\BarberResource;
use App\Http\Resources\UserResource;
use App\Http\Resources\UsersResource;
use App\Models\Address;
use App\Models\Token;
use App\Models\UserToken;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;

class UsersController extends Controller
{
    public function client_register(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'mobile' => ['required', 'unique:users,mobile'],
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android',
            'password' => 'required|confirmed|min:6',
            'password_confirmation' => 'sometimes|required_with:password',
            'name' => 'required',
            'last_name' => 'required',
            'email' => 'required|email|unique:users',
            'lang' => 'required_if:device_type,ios'

        ]);
        if ($validator->passes()) {
            $user = new User();
            $user['mobile'] = $request['mobile'];
            $user['email'] = $request['email'];
            $user['name'] = $request['name'];
            $user['last_name'] = $request['last_name'];
            $user['password'] = bcrypt($request['password']);
            $user['type'] = 'client';
            $user['notifications'] = 'true';
            $user['status'] = 'active';
            $user['admin_accept'] = 'true';
            $user->save();

            $check = UserToken::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                $check->user_id = $user->id;
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check->lang = $request['lang'] ? $request['lang'] : $lang;
                $check->update();
            } else {
                UserToken::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected',
                    'lang' => $request['lang'] ? $request['lang'] : $lang
                ]);
            }

            $data = new UsersResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function barber_register(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'mobile' => ['required', 'unique:users,mobile'],
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android',
            'password' => 'required|confirmed|min:6',
            'password_confirmation' => 'sometimes|required_with:password',
            'name' => 'required',
            'last_name' => 'required',
            'email' => 'required|email|unique:users',
            'country' => 'required',
            'birthday' => 'required',
            'id_image' => 'required|image',
            'image' => 'required|image',
            'town_image' => 'required|image',
            'lat' => 'required',
            'lng' => 'required',
            'lang' => 'required_if:device_type,ios'

        ]);
        if ($validator->passes()) {
            $user = new User();
            $user['mobile'] = $request['mobile'];
            $user['email'] = $request['email'];
            $user['name'] = $request['name'];
            $user['last_name'] = $request['last_name'];
            $user['birthday'] = $request['birthday'];
            $user['lat'] = $request['lat'];
            $user['lng'] = $request['lng'];
            $user['country'] = $request['country'];
            $user['password'] = bcrypt($request['password']);
            $user['type'] = 'barber';
            $user['notifications'] = 'true';
            $user['status'] = 'not_active';
            $user['admin_accept'] = 'false';
            if ($request->hasFile('image')) $user->setImageAttribute();
            if ($request->hasFile('id_image')) {
                $filename = 'id_image' . time() . '.' . $request->id_image->getClientOriginalExtension();
                $request['id_image']->move(public_path('images/users'), $filename);
                $user['id_image'] = $filename;
            }
            if ($request->hasFile('town_image')) {
                $filename = 'town_image' . time() . '.' . $request->town_image->getClientOriginalExtension();
                $request['town_image']->move(public_path('images/users'), $filename);
                $user['town_image'] = $filename;
            }

            $user->save();

            $check = UserToken::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                $check->user_id = $user->id;
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check->lang = $request['lang'] ? $request['lang'] : $lang;

                $check->update();
            } else {
                UserToken::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected',
                    'lang' => $request['lang'] ? $request['lang'] : $lang
                ]);
            }

            $data = new UsersResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }


    public function login(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'email' => 'required',
            'password' => 'required',
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android',
            'type' => 'required|in:client,barber',
            'lang' => 'required_if:device_type,ios'

        ]);
        if ($validator->passes()) {
            $auth = ['email' => $request->input('email'), 'password' => $request->input('password'), 'status' => 'active', 'type' => $request['type'], 'admin_accept' => 'true'];
            try {
                if (!$token = JWTAuth::attempt($auth, ['exp' => Carbon::now()->addDays(7300)->timestamp])) {

                    return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_password')], 422);
                }
            } catch (JWTException $e) {
                return response()->json(['value' => false, 'msg' => 'could_not_create_token'], 500);
            }
            $user = User::whereEmail($request['email'])->first();

            $check = UserToken::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check['user_id'] = $user->id;
                $check->lang = $request['lang'] ? $request['lang'] : $lang;

                $check->update();
            } else {
                UserToken::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected',
                    'lang' => $request['lang'] ? $request['lang'] : $lang,
                ]);
            }

            $data = new UsersResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function change_availability(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'accept_orders' => 'required|in:true,false'
        ]);
        if ($validator->passes()) {
            $user->accept_orders = $request['accept_orders'];
            $user->update();
            $data = new UsersResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

        }
    }

    public function my_addresses()
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $rows = Address::where('user_id', $user->id)->orderByDesc('id')->paginate(20);
        return AddressesResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function delete_address($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        Address::destroy($id);
        $rows = Address::where('user_id', $user->id)->paginate(20);
        return AddressesResource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function logout(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'device_token' => 'required|exists:user_tokens,token'
        ]);
        if ($validator->passes()) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }


            $exist_token = UserToken::where(['token' => $request['device_token']])->first();
            if (!$exist_token) return response()->json(['value' => false, 'msg' => 'device token and auth not match', 'code' => 401]);

            $exist_token->status = 'disconnected';
            $exist_token->update();

            JWTAuth::invalidate(JWTAuth::getToken());
            return response()->json(['value' => true, 'data' => \Lang::get('data.logout'), 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }


    }


    public function forget_password(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required'
        ]);
        if ($validator->passes()) {

            $user = User::whereMobile($request['mobile'])->first();
            if (!$user) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_mobile'), 'code' => 422]);
            $code = rand(1111, 9999);
            $user->code = $code;
            $user->update();
            // mobile code sms
            $message = " عميلنا العزيز الكود الخاص بكم هو $code ";
            shamel_sms($request['mobile'],$message);
            $data = new UsersResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function check_forget_code(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required|exists:users,mobile',
            'code' => 'required|exists:users,code'
        ]);
        if ($validator->passes()) {
            $user = User::whereMobile($request['mobile'])->first();
            if (!$user) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_mobile'), 'code' => 422]);
            if ($user->code != $request['code']) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_code'), 'code' => 422]);
            $data = new UsersResource($user);
            return response()->json(['value' => true, 'data' => $data, 'msg' => \Lang::get('data.correct_code'), 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function change_password(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required|exists:users,mobile',
            'password' => 'required|min:6',
            'device_token' => 'required',
            'device_type' => 'required|in:android,ios',
            'lang' => 'required_if:device_type,ios'

        ]);
        if ($validator->passes()) {

            $user = User::whereMobile($request['mobile'])->first();
            $user->password = bcrypt($request['password']);
            $user->update();
            $check = UserToken::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check['user_id'] = $user->id;
                $check->lang = $request['lang'] ? $request['lang'] : $lang;

                $check->update();
            } else {
                UserToken::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected',
                    'lang' => $request['lang'] ? $request['lang'] : $lang
                ]);
            }
            $data = new UsersResource($user);


            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function barber_info($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $barber = User::find($id);
        $data = new BarberResource($barber);
        return response()->json(['value' => true, 'data' => $data, 'code' => 200]);

    }

    public function update_location(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'lat' => 'required',
            'lng' => 'required'
        ]);
        if ($validator->passes()){
            $user->lat = $request['lat'];
            $user->lng = $request['lng'];
            $user->update();
            $data = new UsersResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function update_profile(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'email' => 'email|unique:users,email,' . $user->id,
            'mobile' => 'unique:users,email,' . $user->id,
            'notifications' => 'in:true,false',
            'image' => 'image'
        ]);
        if ($validator->passes()) {
            $user->name = $request['name'] ? $request['name'] : $user->name;
            $user->last_name = $request['last_name'] ? $request['last_name'] : $user->last_name;
            $user['mobile'] = $request['mobile'] ? $request['mobile'] : $user->mobile;
            $user['country'] = $request['country'] ? $request['country'] : $user->country;
            $user['email'] = $request['email'] ? $request['email'] : $user->email;
            if ($request->hasFile('image')) $user->setImageAttribute();
            $user['notifications'] = $request['notifications'] ? $request['notifications'] : $user->notifications;
            $user->update();
            $data = new UsersResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function change_old_password(Request $request)
    {

        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'old_password' => 'required',
            'new_password' => 'required|confirmed',
            'new_password_confirmation' => 'required'
        ]);
        if ($validator->passes()) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }

            if (\Hash::check($request['old_password'], $user->password)) {
                $user->password = bcrypt($request['new_password']);
                $user->update();
                $data = new UsersResource($user);
                return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
            } else {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.password_not_match'), 'code' => 422]);
            }

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }

    }

    public function update_language(Request $request)
    {

        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'device_token' => 'required',
            'device_type' => 'required|in:android,ios',
            'lang' => 'required_if:device_type,ios'
        ]);
        if ($validator->passes()) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }
            $check = UserToken::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check['user_id'] = $user->id;
                $check->lang = $request['lang'] ? $request['lang'] : $lang;

                $check->update();
            } else {
                UserToken::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected',
                    'lang' => $request['lang'] ? $request['lang'] : $lang
                ]);
            }
            return response()->json(['value' => true, 'data' => 'done', 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }

    }

    public function address(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('wrong token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'address' => 'required',
            'lat' => 'required',
            'lng' => 'required'
        ]);
        if ($validator->passes()) {
            Address::create([
                'user_id' => $user->id,
                'address' => $request['address'],
                'lat' => $request['lat'],
                'lng' => $request['lng']
            ]);
            $data = new UsersResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function getAuthenticatedUser()
    {
        try {

            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['user_not_found'], 404);
            }

        } catch (Tymon\JWTAuth\Exceptions\TokenExpiredException $e) {

            return response()->json(['token_expired'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\TokenInvalidException $e) {

            return response()->json(['token_invalid'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\JWTException $e) {

            return response()->json(['token_absent'], $e->getStatusCode());

        }

        return response()->json(compact('user'));
    }
}
