<?php

namespace App\Http\Resources;

use App\Models\Offer;
use App\Models\Setting;
use Carbon\Carbon;
use Illuminate\Http\Resources\Json\JsonResource;

class BarberOrderResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function toArray($request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if ($lang == 'ar') {
            $visa = "فيزا";
            $cash = "كاش";
        } else {
            $visa = "Visa";
            $cash = "Cash";
        }
        $order_created_at = Carbon::parse($this->created_at);
        $order_after_two = Carbon::parse($this->created_at)->addMinutes(2);

        if (Carbon::now() > $order_after_two) {
            $resend = true;
        } else {
            $resend = false;
        }
        $discount_percent = Offer::where('code', $this->discount_code)->first();
        $setting  = Setting::first();
        $all_price = $this->services_price + $setting->delivery_fee + $setting->fee;
        $discount_amount = 0 ;
        if ( $this->services_price >= 150){

            $discount_amount =   $this->services_price * $setting->discount_fee / 100;

        }
        return [
            'id' => $this->id,
            'user_id' => $this->user_id ? (integer)$this->user_id : 0,
            'barber_id' => $this->barber_id ? (integer)$this->barber_id : 0,
            'services_price' => $this->services_price ? $this->services_price : '',
            'address' => $this->address  ? $this->address->address : '',
            'notes' => $this->notes ? $this->notes : '',
            'payment_type' => ($this->payment_type == 'visa') ? $visa : $cash,
            'discount_amount'=> (string)($discount_amount) ,
            'all_price'=>(integer)$all_price,
            'discount_code' => $this->discount_code ? $this->discount_code : '',
            'discount_percent' => $discount_percent ? $discount_percent->discount : '',
            'total_price' => $this->total_price ? (string)$this->total_price : "",
            'status' => $this->status,
            'order_code' => $this->order_code,
            'order_fees' => $this->fees ? $this->fees : '',
            'deliver_fee'=>$this->delivery_fee ? $this->delivery_fee : '',
            'resend_order' => $resend,
            'barber_name' => $this->barber_id ? $this->barber->name . " ". $this->barber->last_name : '',
            'barber_mobile' => $this->barber_id ? $this->barber->mobile : '',
            'barber_rate' => $this->barber_id ? getRateForUser($this->barber_id) : 0,
            'created_at' => Carbon::parse($this->created_at)->toDateString(),
            'time' => Carbon::parse($this->created_at)->toTimeString(),
            'services' => $this->barber_order_services(),
            'client_name'=>$this->user->name . " " . $this->user->last_name,
            'client_email'=>$this->user->email ? $this->user->email : '' ,
            'client_mobile'=>$this->user->mobile ? $this->user->mobile : '' ,
            'client_lat'=>$this->address ? $this->address->lat : '',
            'client_lng'=>$this->address ? $this->address->lng : '',
            'client_rate'=>getRateForUser($this->user->id)

        ];
    }

}
