<?php

namespace App;

use App\Models\Address;
use App\Models\Contact;
use App\Models\Notification;
use App\Models\Order;
use App\Models\Rate;
use App\Models\Service;
use App\Models\UserToken;
use Carbon\Carbon;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Tymon\JWTAuth\Contracts\JWTSubject;

class User extends Authenticatable implements JWTSubject
{
    use Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array('name', 'email', 'mobile', 'type', 'country', 'password', 'notifications', 'image', 'status', 'admin_accept', 'accept_orders', 'lng', 'code', 'birthday', 'id_image', 'town_image');


    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    public function getJWTIdentifier()
    {
        return $this->getKey();
    }

    public function getJWTCustomClaims()
    {
        return [];
    }

    public function setImageAttribute()
    {
        $rand = rand(111, 999);
        $file = request()->file('image');
        $destinationPath = 'images/users/';
        $filename = $rand . '_' . time() . '.' . $file->getClientOriginalExtension();
        $file->move($destinationPath, $filename);
        $this->attributes['image'] = $filename;
    }


    public function services()
    {
        return $this->belongsToMany(Service::class, 'user_services', 'user_id', 'service_id');
    }

    public function barber_rates($id)
    {
        $lang = request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $data = [];
        $rows = Rate::where('rated_id', $id)->get();
        foreach ($rows as $item) {
            $arr['id'] = $item->id;
            $user = User::find($id);
            $arr['rated_name'] = $user->name . " " . $user->last_name;
            $arr['body'] = $item->body;
            $arr['rate'] = $item->rate;
            $arr['since'] = Carbon::parse($item->created_at)->diffForHumans();
            $data[] = $arr;
        }
        return $data;
    }

    public function addresses()
    {
        return $this->hasMany(Address::class, 'user_id');
    }

    public function notifications()
    {
        return $this->hasMany(Notification::class, 'user_id');
    }

    public function contacts()
    {
        return $this->hasMany(Contact::class, 'user_id');
    }

    public function user_orders()
    {
        return $this->hasMany(Order::class, 'user_id');
    }

    public function barber_orders()
    {
        return $this->hasMany(Order::class, 'barber_id');
    }

    public function tokens()
    {
        return $this->hasMany(UserToken::class, 'user_id');
    }
}
